#!/usr/bin/env python

#    Tichy
#    copyright 2008 Guillaume Chereau (charlie@openmoko.org)
#
#    This file is part of Tichy.
#
#    Tichy is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    Tichy is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Tichy.  If not, see <http://www.gnu.org/licenses/>.


from ..service import Service, ServiceUnusable
from ..object import Object
from tichy.tasklet import Tasklet, Wait, WaitDBus

import logging
logger = logging.getLogger('GSM')

class GSMService(Service):
    """GSM Service base class
    
    signals :
    * provider-modified(string name)
    * incoming-call(call)
    
    """
    def register(self):
        """This must return a Tasklet"""
        raise NotImplementedError
    def create_call(self, number):
        """create a new call to a given number"""
        raise NotImplementedError
        
    class Call(Object):
        """This class represent a call
        
        status:
        * inactive
        * incoming
        * outgoing
        * activating
        * activated
        * releasing
        * released
        
        """
        pass
        
import dbus
from dbus.mainloop.glib import DBusGMainLoop
DBusGMainLoop(set_as_default=True)

import gobject

class FreeSmartPhoneGSM(GSMService):
    service = 'GSM'
    """GSMService that uses freesmartphone DBUS API"""
    singleton = True    # Make the service a singleton
    
    def __init__(self):
        logger.info("connecting to freesmartphone.GSM dbus interface")
        try:
            # We create the dbus interfaces to org.freesmarphone
            bus = dbus.SystemBus()
            self.gsm = bus.get_object( 'org.freesmartphone.ogsmd', '/org/freesmartphone/GSM/Device' )
            self.gsm.connect_to_signal("Status", self.on_status)
            self.gsm.connect_to_signal("CallStatus", self.on_call_status)
        except Exception, e:
            logger.warning("can't use freesmartphone GSM : %s", e)
            self.gsm = None
            raise ServiceUnusable
            
        self.lines = {}
        
    def register(self):
        # TODO: simplify this
        class Process(Tasklet, Object):
            def __init__(self, gsm):
                self.gsm = gsm
                super(Process, self).__init__()
                
            def run(self):
                try:
                    logger.info("Turn on antenna power")
                    self.emit('step', 'Turn on antenna power')
                    yield WaitDBus(self.gsm.gsm.SetAntennaPower, True)
                    self.emit('step', 'Register on the network')
                    logger.info("register on the network")
                    yield WaitDBus(self.gsm.gsm.Register)
                    yield Wait(self.gsm, 'provider-modified')
                except Exception, e:
                    logger.error("Error : %s", e)
                    raise
                    
        return Process(self)

    
    def on_call_status(self, id, status, properties ):
        logger.debug("CALL STATUS %s %s %s", id, status, properties)
        id = int(id)
        status = str(status)
        
        if status == 'incoming':
            if id in self.lines:    # XXX: should use an assert, but it doesn't work on neo :O
                logger.warning("WARNING : line already present %s %s", id, self.lines)
                return  # XXX : I just ignore the message, because the framework send it twice !! Bug in the framework ?
                # raise Exception("call already present")
            peer_number = str(properties.get('peer', "Unknown"))
            call = FreeSmartPhoneGSM.Call(self, peer_number, id)
            assert id in self.lines
            self.emit('incoming-call', call)
        else:
            assert id in self.lines
            self.lines[id].on_call_status(status)
            
    def on_status(self, status):
        logger.debug("STATUS %s", status)
        if 'provider' in status:
            self.emit('provider-modified', str(status['provider']))
            

    def create_call(self, number):
        logger.info("create call %s" % number)
        if not self.gsm:
            raise Exception("No GSMService")    # TODO : use a real exception
        # id = self.gsm.Initiate(number, "voice")
        return FreeSmartPhoneGSM.Call(self, number)


    class Call(GSMService.Call):
        def __init__(self, service, number, id = None):
            super(FreeSmartPhoneGSM.Call, self).__init__()
            self.service = service
            self.number = number
            self.id = id  # The id used by freesmartphone API
            if id is not None:
                self.service.lines[id] = self
                self.status = 'incoming'
            else:
                self.status = 'inactive'
            
        def initiate(self):
            self.id = int(self.service.gsm.Initiate(self.number, "voice"))
            # Don't forget to register into the gsm service
            self.service.lines[self.id] = self
            
        def release(self):
            # It is ok top call release even if the call is already inactive
            if self.status not in ['releasing', 'released']:
                self.service.gsm.Release(self.id)
            self.status = 'releasing'
        
        def activate(self):
            assert self.id is not None
            self.service.gsm.Activate(self.id)
            self.status = 'activating'
            
        def on_call_status(self, status):
            if status == 'outgoing':
                self.status = 'outgoing'
                self.emit('outgoing')
            elif status == 'active':
                self.status = 'active'
                self.emit('activated')
            elif status == 'release':
                self.status = 'released'
                del self.service.lines[self.id]
                self.id = None
                self.emit('released')
                
        
class TestGsm(Service):
    """This is just a fake service that can be use to test without GSM drivers"""
    service = 'GSM'
    class Call(Object):
        def __init__(self, number):
            super(TestGsm.Call, self).__init__()
            self.number = number
            self.status = 'inactive'
            print 'Using fake GSM service'
        def release(self):
            self.status = 'released'
            gobject.timeout_add(10, self.emit, 'released')
        def initiate(self):
            gobject.timeout_add(1000, self.emit, 'activated')
        def destroy(self):
            pass
            
    def register(self):
        class Process(Tasklet, Object):
            def __init__(self, gsm):
                self.gsm = gsm
                super(Process, self).__init__()
            def run(self):
                self.emit('step', 'Turn on antenna power')
                self.emit('step', 'Register on the network')
                self.gsm.emit('provider-modified', 'Charlie Telecom')
                yield None
                
        return Process(self)
            
    def __init__(self):
        self.lines = {}
    
    def create_call(self, number):
        logger.debug("test initiate")
        return TestGsm.Call(number)
        

