#    Tichy
#    copyright 2008 Guillaume Chereau (charlie@openmoko.org)
#
#    This file is part of Tichy.
#
#    Tichy is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    Tichy is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Tichy.  If not, see <http://www.gnu.org/licenses/>.

from object import Object

from item import Item, ItemMetaClass

import logging
logger = logging.getLogger('Service')

class BaseService(Item):
    def __init__(self, service):
        self.service = service


class ServiceMetaClass(ItemMetaClass):
    """The meta class for Service class"""
    def __init__(cls, name, bases, dict):
        if 'service' in dict:
            service_name = dict['service']
            if '__init__' in dict:
                cls._Service__init = dict['__init__']
                cls.__init__ = Service.__init__
            logger.debug("Register %s as service %s", cls, service_name) 
            Service._Service__all_services.setdefault(service_name, {})[cls.name] = cls
            if service_name not in Service._Service__bases:
                Service._Service__bases[service_name] = BaseService(service_name)
            cls.base = Service._Service__bases[service_name]
        ItemMetaClass.__init__(cls, name, bases, dict)
        
class ServiceUnusable(Exception):
    def __init__(self, msg = None):
        super(ServiceUnusable, self).__init__(msg)

class Service(Item):
    __metaclass__ = ServiceMetaClass
    # This attribute is a dict of dict : (service_name -> (name -> service_cls))
    __all_services = {}
    # A dict of the form (service_name -> service)
    __defaults = {}
    # Every service has a base service object associated
    # This is used to have a common object associated with all service of the same group
    # This object is put into the `base` attribute of every service classes
    __bases = {}
    
    # This is used to store the sigleton value of the service
    __singleton = None
    
    enabled = True
    name = None
    
    @classmethod
    def set_default(cls, service, default):
        """Set a given service as the default one"""
        if isinstance(default, str):
            default = Service(service, default)
        cls.__defaults[service] = default
        Service.__bases[service].emit("changed", default)
        
    @classmethod
    def get_all(cls, name):
        ret = []
        for service in Service.__all_services.get(name, {}).values():
            try:
                instance = service.__get()
                if not instance.enabled:
                    continue
                ret.append(instance)
            except ServiceUnusable:
                logger.warning("service %s unusable, skipped", service)
        return ret

    def __new__(cls, service = None, name = None):
        if service is None:
            return object.__new__(cls)
            
        if name:
            all_services = [Service.__all_services[service][name]]
        else:
            all_services = Service.__all_services.get(service, {}).values()
            if service in cls.__defaults:
                all_services = [cls.__defaults[service]] + all_services
            
        for service_cls in all_services:
            logger.debug("try to get service %s", service)
            try:
                ret = service_cls.__get()
                if not ret.enabled:
                    continue
                return ret
            except ServiceUnusable:
                logger.warning("service %s unusable, skipped", service_cls)
        raise KeyError("can't find any service '%s'" % name)
        
    def __init__(self, service = None, name = None):
        super(Service, self).__init__()
        
    def __init(self):
        pass

    @classmethod
    def __get(cls):
        if cls.__singleton is None:
            singleton = cls()
            singleton.__init()
            cls.__singleton = singleton
        return cls.__singleton


        
